//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//////////////////////////////////////////////////////////////////////
// NesMapper5.cpp - Implementation of mapper #5 (MMC5)
//////////////////////////////////////////////////////////////////////
//
// This code is in its very early stages and doesn't do hardly
// anything yet
//

#include "NesMapper005.h"



NesMapper5::NesMapper5(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	// Replace the standard PPU memory object with a VRC4 specific one	
	NesPPU *newPPU = new MMC5PPU();
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);
	

	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;

	intPRGSize = 0;
	intCHRSize = 0;
	
	initMaps();
	blnIrqEnable = false;
	intIrqCount = 0;
	intIrqLine = 0;
	intIrqScanline = 0;
	intIrqStatus = 0;
	intIrqClear = 0;
}


// On reset, the last 8K PRG-ROM is swapped into all 4 slots
void NesMapper5::initMemory()
{	
	if (bytPRGCount > 0) {
		swapPRGBank8(0x8000, bytPRGCount*2 - 1 );
		swapPRGBank8(0xA000, bytPRGCount*2 - 1 );
		swapPRGBank8(0xC000, bytPRGCount*2 - 1 );
		swapPRGBank8(0xE000, bytPRGCount*2 - 1 );
	}
	if (bytCHRCount) {
		swapCHRBank8(0);
		for (int i = 0; i < 8; i++) {
			CHRMap[i] = i;
		}
	}
}


void NesMapper5::write(Uint16 addr, unsigned char data) {			
	switch (addr) {
	
	case 0x5100:	// Select PRG-ROM page size
		// 0 = 32k, 1 = 16k, 2,3 = 8k
		intPRGSize = data & 0x03;
		break;

	case 0x5101:	// Sprite CHR Bank size
		intCHRSize = data & 0x03;
		break;

	case 0x5104:	// EXRAM register
		break;
	
	case 0x5105:	// Nametable select
		break;

	//////////////////////////// PRGROM Select ///////////////////////////////
	case 0x5114:	
		if (data & 0x80) {
			// Select low 8K PRGROM
			if (intPRGSize == 3) swapPRGBank8(0x8000, (data & 0x7F) % (bytPRGCount << 1));
		} else {
			// Select SaveRAM
		}
		break;

	case 0x5115:
		if (data & 0x80) {
			// Select next lowest 8/16k PRGROM
			if ((intPRGSize == 1) || (intPRGSize == 2) ) {
				swapPRGBank16(0x8000, ((data & 0x7F) >> 1) % bytPRGCount);
			} else if (intPRGSize == 3) {
				swapPRGBank8(0xA000, (data & 0x7F) % (bytPRGCount << 1));
			}
		} else {

		}
		break;
	
	case 0x5116:
		if (data & 0x80) {
			// Select 8K PRGROM at $C000
			if ((intPRGSize == 2) || (intPRGSize == 3) ) swapPRGBank8(0xC000, (data & 0x7F) % (bytPRGCount << 1));			
		} else {

		}
		break;
	
	case 0x5117:	// Select PRGROM at highest slot for respective bank size
		if (data & 0x80) {
			switch (intPRGSize) {
			case 0:
				swapPRGBank32( ((data & 0x7F) >> 2) % (bytPRGCount >> 1) );
				break;
			case 1:
				swapPRGBank16(0xC000, ((data & 0x7F) >> 1) % bytPRGCount );
				break;
			case 2:
			case 3:
				swapPRGBank8(0xE000, (data & 0x7F) % (bytPRGCount << 1) );
				break;
			}						
		} else {

		}
		break;
	//////////////////////////////////////////////////////////////////////////////////
	

	//////////////////////////// CHRROM Select ///////////////////////////////
	case 0x5120:	// Select VROM at $0000		
	case 0x5121:	// Select low VROM (1k or 2k)		
	case 0x5122:	// Select 1k VROM at $0800	
	case 0x5123:	// Select VROM at $C00/$800		
	case 0x5124:	// Select 1k VROM at $1000					
	case 0x5125:	// Select VROM at $1400/$1000				
	case 0x5126:	// Select 1k VROM at $1800			
	case 0x5127:	// Select VROM at $1C00/$1800/$1000/$0000
		CHRMap[addr & 0x07] = data;
		setCHRROMPages(addr, data);
		break;
	


	case 0x5128:	// Select 2k VROM at $0000
		break;

	case 0x5129:	// Select 2k VROM at $800
		break;

	case 0x512A:	// Select 2k VROM at $1000
		break;
	
	case 0x512B:	// Select 2k VROM at $1800
		break;
	//////////////////////////////////////////////////////////////////////////////////
	
	case 0x5203:	// IRQ
		intIrqLine = data;
		break;

	case 0x5204:	// IRQ
		blnIrqEnable = (data & 0x80) ? true : false;
		break;
		
	}
}

Uint8 NesMapper5::read(Uint16 addr) {
	Uint8 retval;
	switch (addr) {
	case 0x5204:
		retval = intIrqStatus;
		intIrqStatus &= ~0x80;
		break;

	default:
		retval = mCpu->memory[addr];
		break;
	}
	return retval;
}



// This sets all CHRROM pages for writes to $5120-$5127
void NesMapper5::setCHRROMPages(unsigned char addr, unsigned char data) {
	switch (intCHRSize) {
	case 0:		// Select 8K
		swapCHRBank8( CHRMap[7] % bytCHRCount);
		break;

	case 1:		// Select 4K
		swapCHRBank4(0x0000, CHRMap[3] % (bytCHRCount << 1) );
		swapCHRBank4(0x1000, CHRMap[7] % (bytCHRCount << 1) );
		break;

	case 2:		// Select 2K
		swapCHRBank2(0x0000, CHRMap[1] % (bytCHRCount << 2) );
		swapCHRBank2(0x0800, CHRMap[3] % (bytCHRCount << 2) );
		swapCHRBank2(0x1000, CHRMap[5] % (bytCHRCount << 2) );
		swapCHRBank2(0x1800, CHRMap[7] % (bytCHRCount << 2) );
		break;	

	case 3:		// Select 1K
		for (int i = 0; i < 8; i++) {
			swapCHRBank1(0x0400 * i, CHRMap[i] % (bytCHRCount << 3) );
		}
		break;
	}
}