//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper010.cpp - Implementation of mapper #10 (MMC4)
//////////////////////////////////////////////////////////////////////

#include "NesMapper010.h"


NesMapper10::NesMapper10(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;

	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;

	intLatchReg1 = intLatchReg2 = 0;

	// Replace the standard PPU memory object with a MMC2 specific one	
	NesPPUMemory *newPPUMemory = new MMC2PPUMemory();
	newPPUMemory->link(this);
	newPPUMemory->init();		
	newPPUMemory->mintLatchValue1 = 0;
	newPPUMemory->mintLatchValue2 = 0;
	delete mPpu->VRAM;
	mPpu->VRAM = newPPUMemory;
	
	initMaps();
}


NesMapper10::~NesMapper10()
{
}

// First 16K PRGROM bank is laoded into $8000, last 16k PRGROM bank into $C000
void NesMapper10::initMemory()
{
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank ot $C000
	}

	if (bytCHRCount) swapCHRBank8(0);	
}


void NesMapper10::write(Uint16 addr, unsigned char data) {					
	switch (addr & 0xF000) {
	
	case 0xA000: // $A000 - $AFFF: Select 16k PRGROM at $8000
		swapPRGBank16(0x8000, (data % bytPRGCount));		
		break;
		
	case 0xB000: // $B000 - $BFFF: Select 4k VROM at PPU $0000 when latch is $FD						
		if (mPpu->VRAM->mintLatchSelect1 == 0xFD) {						
			if (mPpu->VRAM->mintLatchValue1 != data) swapCHRBank4(0x0000, data);
		}
		mPpu->VRAM->mintLatchValue1 = data;
		break;

	case 0xC000: // $C000 - $CFFF: Select 4k VROM at PPU $0000 when latch is $FE						
		if (mPpu->VRAM->mintLatchSelect1 == 0xFE) {			
			if (mPpu->VRAM->mintLatchValue2 != data) swapCHRBank4(0x0000, data);			
		}
		mPpu->VRAM->mintLatchValue2 = data;
		break;
		
	case 0xD000: // $D000 - $DFFF: Select 4k VROM bank at PPU $1000 when latch selector is $FD
		if (mPpu->VRAM->mintLatchSelect2 == 0xFD) {					
			if (mPpu->VRAM->mintLatchValue3 != data) swapCHRBank4(0x1000, data);			
		}
		mPpu->VRAM->mintLatchValue3 = data;
		break;
	
	case 0xE000: // $E000 - $EFFF: Select 4k VROM bank at PPU $1000 when latch selector is $FE						
		//if (data == 0x6) data = 0x7;
		if (mPpu->VRAM->mintLatchSelect2 == 0xFE) {						
			if (mPpu->VRAM->mintLatchValue4 != data) swapCHRBank4(0x1000, data);			
		}
		mPpu->VRAM->mintLatchValue4 = data;
		break;
	//////////////////////////////

	case 0xF000: // $F000 - $FFFF: Mirroring select
		if (data & 0x01) {
			// Horizontal mirroring
			mPpu->VRAM->mbytMirroringType = HORIZONTAL;			
		} else {
			// Vertical mirroring
			mPpu->VRAM->mbytMirroringType = VERTICAL;
		}		
		break;
	}
}
