//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper11.cpp - Implementation of mapper #11 (Color Dreams)
//////////////////////////////////////////////////////////////////////

#include "NesMapper011.h"



NesMapper11::NesMapper11(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU)
{
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


void NesMapper11::initMemory()
{	
	if (bytPRGCount > 0) {
		swapPRGBank32(0);			  // First 32k bank to $8000
	}
	if (bytCHRCount > 0) {
		swapCHRBank8(0);			  // First 8k VROM bank to $8000
	}

}


void NesMapper11::write(Uint16 addr, unsigned char data)
{	
	// Select 32k PRGROM, 16k VROM	
	if (bytPRGCount) swapPRGBank32( (data & 0x0F) % (bytPRGCount>>1));
	if (bytCHRCount) swapCHRBank8( ((data & 0xF0) >> 4) % (bytCHRCount));
}