//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper228.cpp - Implementation of mapper #228 (Action 52)
// This implementation was written from looking at the VirtuaNES source
//////////////////////////////////////////////////////////////////////

#include "NesMapper228.h"


NesMapper228::NesMapper228(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


void NesMapper228::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank32(0);					  // First 32k bank to $8000
	}
	if (bytCHRCount) swapCHRBank8(0);		  // First 8K CHRROM to VRAM
}


void NesMapper228::write(Uint16 addr, unsigned char data) {		
	// On writes over $8000...
	if (addr & 0x8000) {
		
		// PRG select low is bits 7-11
		unsigned char prg = (addr & 0x0780) >> 7;
	
		// PRG select high
		if ((addr & 0x1800) == 0x0800) prg |= 0x10;
		if ((addr & 0x1800) == 0x1800) prg |= 0x20;

		// Swap in PRG
		if (addr & 0x0020) {
			prg <<= 1;
			if (addr & 0x0040) {
				prg++;
			}
			swapPRGBank8(0x8000, (4*prg + 0) % (bytPRGCount << 1) );
			swapPRGBank8(0xA000, (4*prg + 1) % (bytPRGCount << 1) );
			swapPRGBank8(0xC000, (4*prg + 0) % (bytPRGCount << 1) );
			swapPRGBank8(0xE000, (4*prg + 1) % (bytPRGCount << 1) );
		} else {
			swapPRGBank8(0x8000, (4*prg + 0) % (bytPRGCount << 1) );
			swapPRGBank8(0xA000, (4*prg + 1) % (bytPRGCount << 1) );
			swapPRGBank8(0xC000, (4*prg + 2) % (bytPRGCount << 1) );
			swapPRGBank8(0xE000, (4*prg + 3) % (bytPRGCount << 1) );
		}

		// Lower 4 bits of addr + lower 2 bits of data = CHR select
		swapCHRBank8(  (((addr & 0xF) << 2) | (data & 0x03)) % bytCHRCount );
		
		// Bit 14 = mirroring select
		if (addr & 0x2000) {
			mPpu->VRAM->mbytMirroringType = HORIZONTAL;
		} else {
			mPpu->VRAM->mbytMirroringType = VERTICAL;
		}
	}
	
	
}