//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


// --------------------------------------------------------------------
// BaseSoundChip.h
// --------------------------------------------------------------------
// This class defines the interface all expansion chips (e.g. Konami VRC6) must follow.
// The main requirements are:
// Functions: write(), exec(), reset(), getCurrentSample(), setCurrentSample(), loadState(), saveState()
// Data nembers: intCurSample
// --------------------------------------------------------------------


#ifndef SOUNDCHIPHEADER
#define SOUNDCHIPHEADER

#include "../Porting/basetypes.h"

class BaseSoundChip {
public:
	BaseSoundChip() { };
	~BaseSoundChip() { };
	
	virtual void exec(int intCycles) { };
	virtual void write(Uint16 addr, unsigned char data) { };
	virtual void reset() { };
	
	virtual void saveState(FILE *stream) { };
	virtual void loadState(FILE *stream) { };

	virtual unsigned char getCurrentSample() { return 0; };
	virtual void setCurrentSample(int intNewSample) { intCurSample = intNewSample; };

	int intCurSample;

private:

};

#endif