//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


// --------------------------------------------------------------------
// VRC6.h
// --------------------------------------------------------------------
// This class emualtes the Konami VRC6 sound chip, which consists of
// 2 more square channels plus a sawtooth channel
// --------------------------------------------------------------------

#ifndef VRC6HEADER
#define VRC6HEADER

#include "BaseSoundChip.h"

class VRC6 : public BaseSoundChip {
public:
	struct VRC6Pulse {
		int intDutyType;
		int intVolume;
		int intGateType;
		int intFrequency;
		int intChannelEnable;

		int intCycleCtr;
		int intDutyCtr;
		int intCurSample;
	};

	struct VRC6Sawtooth {		
		int intPhase;
		int intFrequency;
		int intChannelEnable;

		int intAccum;
		int intAccumCtr;
		int intCycleCtr;
		int intCurSample;
	};

	VRC6Pulse pulse1, pulse2;
	VRC6Sawtooth sawtooth;

	VRC6();
	~VRC6() { };
	
	void exec(int intCycles);
	void write(Uint16 addr, unsigned char data);
	void reset();

	void saveState(FILE *stream);
	void loadState(FILE *stream);

	unsigned char getCurrentSample() { return (char) intCurSample; };

	int intCurSample;

private:

};

#endif